@@ -1,5 +1,5 @@
 package Git::Repository;
-
+$Git::Repository::VERSION = '1.314';
 use warnings;
 use strict;
 use 5.006;
@@ -11,8 +11,6 @@ use Scalar::Util qw( looks_like_number );
 
 use Git::Repository::Command;
 
-our $VERSION = '1.25';
-
 # a few simple accessors
 for my $attr (qw( git_dir work_tree options )) {
     no strict 'refs';
@@ -20,8 +18,12 @@ for my $attr (qw( git_dir work_tree options )) {
 }
 
 # backward compatible aliases
-*repo_path = \&git_dir;
-*wc_path   = \&work_tree;
+sub repo_path {
+    croak "repo_path() is obsolete, please use git_dir() instead";
+}
+sub wc_path {
+    croak "wc_path() is obsolete, please use work_tree() instead";
+}
 
 # helper function
 sub _abs_path {
@@ -61,16 +63,27 @@ sub new {
     my $self = bless {}, $class;
 
     # take out the option hash
-    my %arg = grep { !( ref eq 'HASH' ? $self->{options} ||= $_ : 0 ) } @arg;
-    my $options = $self->{options} ||= {};
+    my ( $options, %arg );
+    {
+        my @o;
+        %arg = grep !( ref eq 'HASH' ? push @o, $_ : 0 ), @arg;
+        croak "Too many option hashes given: @o" if @o > 1;
+        $options = $self->{options} = shift @o || {};
+    }
 
-    # ignore 'input' option during object creation
+    # ignore 'input' and 'fatal' options during object creation
     my $input = delete $options->{input};
+    my $fatal = delete $options->{fatal};
+
+    # die if deprecated parameters are given
+    croak "repository is obsolete, please use git_dir instead"
+        if defined delete $arg{repository};
+    croak "working_copy is obsolete, please use work_tree instead"
+        if defined delete $arg{working_copy};
 
     # setup default options
-    # accept older for backward compatibility
-    my ($git_dir) = grep {defined} delete @arg{qw( git_dir repository )};
-    my ($work_tree) = grep {defined} delete @arg{qw( work_tree working_copy )};
+    my $git_dir   = delete $arg{git_dir};
+    my $work_tree = delete $arg{work_tree};
 
     croak "Unknown parameters: @{[keys %arg]}" if keys %arg;
 
@@ -144,32 +157,14 @@ sub new {
 
     # put back the ignored option
     $options->{input} = $input if defined $input;
+    $options->{fatal} = $fatal if defined $fatal;
 
     return $self;
 }
 
+# create() is now fully deprecated
 sub create {
-    my ( $class, @args ) = @_;
-    my @output = $class->run(@args);
-    my $gitdir;
-
-    # create() is now deprecated
-    carp "create() is deprecated, please use run() instead";
-
-    # git init or clone until v1.7.1 (inclusive)
-    if ( $output[0] =~ /^(?:Reinitialized existing|Initialized empty) Git repository in (.*)/ ) {
-        $gitdir = $1;
-    }
-
-    # git clone after v1.7.1
-    elsif ( $output[0] =~ /Cloning into (bare repository )?(.*)\.\.\./ ) {
-        $gitdir = $1 ? $2 : File::Spec->catdir( $2, '.git' );
-    }
-
-    # some other command (no git repository created)
-    else {return}
-
-    return $class->new( git_dir => $gitdir, grep { ref eq 'HASH' } @args );
+    croak "create() is deprecated, see Git::Repository::Tutorial for better alternatives";
 }
 
 #
@@ -187,12 +182,18 @@ sub command {
 sub run {
     my ( $self, @cmd ) = @_;
 
+    # split the args to get the optional callbacks
+    my @cb;
+    @cmd = grep { ref eq 'CODE' ? !push @cb, $_ : 1 } @cmd;
+
+    local $Carp::CarpLevel = 1;
+
     # run the command (pass the instance if called as an instance method)
     my $command
         = Git::Repository::Command->new( ref $self ? $self : (), @cmd );
 
     # return the output or die
-    return $command->final_output;
+    return $command->final_output(@cb);
 }
 
 #
@@ -209,18 +210,10 @@ sub version {
             =~ /git version (.*)/g )[0];
 }
 
-sub _version_eq {
-    my ( $v1, $v2 ) = @_;
-    my @v1 = split /\./, $v1;
-    my @v2 = split /\./, $v2;
-
-    return '' if @v1 != @v2;
-    $v1[$_] ne $v2[$_] and return '' for 0 .. $#v1;
-    return 1;
-}
-
 sub _version_gt {
     my ( $v1, $v2 ) = @_;
+    s/(?<=^1\.0\.)0([ab])$/$1^"P"/e for $v1, $v2; # aliases
+
     my @v1 = split /\./, $v1;
     my @v2 = split /\./, $v2;
 
@@ -248,13 +241,17 @@ sub _version_gt {
 
 # every op is a combination of eq and gt
 sub version_eq {
-    my ( $r, $v, @o ) = ( shift, ( grep !ref, @_ )[0], grep ref, @_ );
-    return _version_eq( $r->version(@o), $v );
+    my ( $r, $v2, @o ) = ( shift, ( grep !ref, @_ )[0], grep ref, @_ );
+    my $v1 = $r->version(@o);
+    s/(?<=^1\.0\.)0([ab])$/$1^"P"/e for $v1, $v2; # aliases
+    return $v1 eq $v2;
 }
 
 sub version_ne {
-    my ( $r, $v, @o ) = ( shift, ( grep !ref, @_ )[0], grep ref, @_ );
-    return !_version_eq( $r->version(@o), $v );
+    my ( $r, $v2, @o ) = ( shift, ( grep !ref, @_ )[0], grep ref, @_ );
+    my $v1 = $r->version(@o);
+    s/(?<=^1\.0\.)0([ab])$/$1^"P"/e for $v1, $v2; # aliases
+    return $v1 ne $v2;
 }
 
 sub version_gt {
@@ -268,27 +265,37 @@ sub version_le {
 }
 
 sub version_lt {
-    my ( $r, $v, @o ) = ( shift, ( grep !ref, @_ )[0], grep ref, @_ );
-    my $V = $r->version(@o);
-    return !_version_eq( $V, $v )
-        && !_version_gt( $V, $v );
+    my ( $r, $v2, @o ) = ( shift, ( grep !ref, @_ )[0], grep ref, @_ );
+    my $v1 = $r->version(@o);
+    s/(?<=^1\.0\.)0([ab])$/$1^"P"/e for $v1, $v2; # aliases
+    return $v1 ne $v2
+        && !_version_gt( $v1, $v2 );
 }
 
 sub version_ge {
-    my ( $r, $v, @o ) = ( shift, ( grep !ref, @_ )[0], grep ref, @_ );
-    my $V = $r->version(@o);
-    return _version_eq( $V, $v )
-        || _version_gt( $V, $v );
+    my ( $r, $v2, @o ) = ( shift, ( grep !ref, @_ )[0], grep ref, @_ );
+    my $v1 = $r->version(@o);
+    s/(?<=^1\.0\.)0([ab])$/$1^"P"/e for $v1, $v2; # aliases
+    return $v1 eq $v2
+        || _version_gt( $v1, $v2 );
 }
 
 1;
 
+# ABSTRACT: Perl interface to Git repositories
+
 __END__
 
+=pod
+
 =head1 NAME
 
 Git::Repository - Perl interface to Git repositories
 
+=head1 VERSION
+
+version 1.314
+
 =head1 SYNOPSIS
 
     use Git::Repository;
@@ -299,6 +306,9 @@ Git::Repository - Perl interface to Git repositories
     # start from an existing working copy
     $r = Git::Repository->new( work_tree => $dir );
 
+    # start from a repository reachable from the current directory
+    $r = Git::Repository->new();
+
     # or init our own repository first
     Git::Repository->run( init => $dir, ... );
     $r = Git::Repository->new( work_tree => $dir );
@@ -307,15 +317,24 @@ Git::Repository - Perl interface to Git repositories
     Git::Repository->run( clone => $url, $dir, ... );
     $r = Git::Repository->new( work_tree => $dir );
 
-    # run commands
-    # - get the full output (no errput)
-    $output = $r->run(@cmd);
+    # provide an option hash for Git::Repository::Command
+    # (see Git::Repository::Command for all available options)
+    $r = Git::Repository->new( ..., \%options );
 
-    # - get the full output as a list of lines (no errput)
-    @output = $r->run(@cmd);
+    # run commands
+    # - get the full output (no errput) passing options for this command only
+    $output = $r->run( @cmd, \%options );
+
+    # - get the full output as a list of lines (no errput), with options
+    @output = $r->run( @cmd, \%options );
+
+    # - process the output with callbacks
+    $output = $r->run( @cmd, sub {...} );
+    @output = $r->run( @cmd, sub {...} );
 
     # - obtain a Git::Repository::Command object
-    $cmd = $r->command(@cmd);
+    #   (see Git::Repository::Command for details)
+    $cmd = $r->command( @cmd, \%options );
 
     # obtain version information
     my $version = $r->version();
@@ -327,12 +346,12 @@ Git::Repository - Perl interface to Git repositories
 
 =head1 DESCRIPTION
 
-C<Git::Repository> is a Perl interface to Git, for scripted interactions
+L<Git::Repository> is a Perl interface to Git, for scripted interactions
 with repositories. It's a low-level interface that allows calling any Git
 command, whether I<porcelain> or I<plumbing>, including bidirectional
 commands such as C<git commit-tree>.
 
-A C<Git::Repository> object simply provides context to the git commands
+A L<Git::Repository> object simply provides context to the git commands
 being run. It is possible to call the  C<command()> and C<run()> methods
 against the class itself, and the context (typically I<current working
 directory>) will be obtained from the options and environment.
@@ -342,23 +361,23 @@ commands. Specifically, it will not prepare environment variables that
 individual Git commands may need or use.
 
 However, the C<GIT_DIR> and C<GIT_WORK_TREE> environment variables are
-special: if the command is run in the context of a C<Git::Repository>
+special: if the command is run in the context of a L<Git::Repository>
 object, they will be overridden by the object's C<git_dir> and
 C<work_tree> attributes, respectively. It is however still possible to
 override them if necessary, using the C<env> option.
 
-C<Git::Repository> requires at least Git 1.5.0, and is expected to support
+L<Git::Repository> requires at least Git 1.5.0, and is expected to support
 any later version.
 
-See C<Git::Repository::Tutorial> for more code examples.
+See L<Git::Repository::Tutorial> for more code examples.
 
-=head1 CONSTRUCTORS
+=head1 CONSTRUCTOR
 
-There are two ways to create C<Git::Repository> objects:
+=head2 new
 
-=head2 new( %args, $options )
+    Git::Repository->new( %args, $options );
 
-Create a new C<Git::Repository> object, based on an existing Git repository.
+Create a new L<Git::Repository> object, based on an existing Git repository.
 
 Parameters are:
 
@@ -376,36 +395,44 @@ is accepted in place of C<git_dir> (but the newer name takes precedence).
 The location of the git working copy (for a non-bare repository).
 
 If C<work_tree> actually points to a subdirectory of the work tree,
-C<Git::Repository> will automatically recompute the proper value.
+L<Git::Repository> will automatically recompute the proper value.
 
 For backward compatibility with versions 1.06 and before, C<working_copy>
 is accepted in place of C<work_tree> (but the newer name takes precedence).
 
 =back
 
-At least one of the two parameters is required. Usually, one is enough,
-as C<Git::Repository> can work out where the other directory (if any) is.
+If none of the parameter is given, L<Git::Repository> will find the
+appropriate repository just like Git itself does. Otherwise, one of
+the parameters is usually enough,
+as L<Git::Repository> can work out where the other directory (if any) is.
 
 C<new()> also accepts a reference to an option hash which will be used
-as the default by C<Git::Repository::Command> when working with the
-corresponding C<Git::Repository> instance.
+as the default by L<Git::Repository::Command> when working with the
+corresponding L<Git::Repository> instance.
 
 So this:
 
-    my $options = {
-        git => '/path/to/some/other/git',
-        env => {
-            GIT_COMMITTER_EMAIL => 'book@cpan.org',
-            GIT_COMMITTER_NAME  => 'Philippe Bruhat (BooK)',
-        },
-    };
     my $r = Git::Repository->new(
+        # parameters
         work_tree => $dir,
-        $options
+        # options
+        {   git => '/path/to/some/other/git',
+            env => {
+                GIT_COMMITTER_EMAIL => 'book@cpan.org',
+                GIT_COMMITTER_NAME  => 'Philippe Bruhat (BooK)',
+            },
+        }
     );
 
 is equivalent to explicitly passing the option hash to each
-C<run()> or C<command()>.
+C<run()> or C<command()> call.
+The documentation for L<Git::Repository::Command> lists all
+available options.
+
+Note that Git::Repository and L<Git::Repository::Command> take
+great care in finding the option hash wherever it may be in C<@_>,
+and to merge multiple option hashes if more than one is provided.
 
 It probably makes no sense to set the C<input> option in C<new()>,
 but L<Git::Repository> won't stop you.
@@ -413,77 +440,74 @@ Note that on some systems, some git commands may close standard input
 on startup, which will cause a C<SIGPIPE>. L<Git::Repository::Command>
 will raise an exception.
 
-=head2 create( @cmd )
-
-B<The C<create()> method is deprecated, and will go away in the future.>
-
-Runs a repository initialization command (like C<init> or C<clone>) and
-returns a C<Git::Repository> object pointing to it. C<@cmd> may contain
-a hashref with options (see L<Git::Repository::Command>.
-
-Do not use the I<-q> option on such commands. C<create()> needs to parse
-their output to find the path to the repository.
-
-C<create()> also accepts a reference to an option hash which will be
-used to set up the returned C<Git::Repository> instance.
-
-Now that C<create()> is deprecated, instead of:
-
-    $r = Git::Repository->create( ... );
-
-simply do it in two steps:
+To create a Git repository and obtain a L<Git::Repository> object
+pointing to it, simply do it in two steps:
 
+    # run a clone or init command without an instance,
+    # using options like cwd
     Git::Repository->run( ... );
+    
+    # obtain a Git::Repository instance
+    # on the resulting repository
     $r = Git::Repository->new( ... );
 
-
 =head1 METHODS
 
-C<Git::Repository> supports the following methods:
+=for Pod::Coverage     create
+    repo_path
+    wc_path
 
-=head2 command( @cmd )
+L<Git::Repository> supports the following methods:
 
-Runs the git sub-command and options, and returns a C<Git::Repository::Command>
+=head2 command
+
+    Git::Repository->command( @cmd );
+    $r->command( @cmd );
+
+Runs the git sub-command and options, and returns a L<Git::Repository::Command>
 object pointing to the sub-process running the command.
 
 As described in the L<Git::Repository::Command> documentation, C<@cmd>
 may also contain a hashref containing options for the command.
 
-=head2 run( @cmd )
+=head2 run
+
+    Git::Repository->run( @cmd );
+    $r->run( @cmd );
 
 Runs the command and returns the output as a string in scalar context,
 or as a list of lines in list context. Also accepts a hashref of options.
 
 Lines are automatically C<chomp>ed.
 
-If the git command printed anything on stderr, it will be printed as
-warnings. If the git sub-process exited with status C<128> (fatal error),
-or C<129> (usage message), C<run()> will C<die()>.
+In addition to the options hashref supported by L<Git::Repository::Command>,
+the parameter list can also contain code references, that will be applied
+successively to each line of output. The line being processed is in C<$_>,
+but the coderef must still return the result string (like C<map>).
 
-=head2 git_dir()
+If the git command printed anything on stderr, it will be printed as
+warnings. For convenience, if the git sub-process exited with status
+C<128> (fatal error), or C<129> (usage message), C<run()> will C<die()>.
+The exit status values for which C<run()> dies can be modified using
+the C<fatal> option (see L<Git::Repository::Command> for details).
+
+The exit status of the command that was just run is accessible as usual
+using C<<< $? >> 8 >>>. See L<perlvar> for details about C<$?>.
+
+=head2 git_dir
 
 Returns the repository path.
 
-=head2 repo_path()
-
-For backward compatibility with versions 1.06 and before, C<repo_path()>
-it provided as an alias to C<git_dir()>.
-
-=head2 work_tree()
+=head2 work_tree
 
 Returns the working copy path.
-Used as current working directory by C<Git::Repository::Command>.
+Used as current working directory by L<Git::Repository::Command>.
 
-=head2 wc_path()
-
-For backward compatibility with versions 1.06 and before, C<wc_path()>
-it provided as an alias to C<work_tree()>.
-
-=head2 options()
+=head2 options
 
 Return the option hash that was passed to C<< Git::Repository->new() >>.
 
-=head2 version()
+=head2 version
 
 Return the version of git, as given by C<git --version>.
 
@@ -519,11 +543,15 @@ The methods are:
 
 All those methods also accept an option hash, just like the others.
 
-Note that there are a small number of cases where the version comparison
-operators will I<not> compare versions correctly for I<very old> versions of
-Git. Typical example is C<1.0.0a gt 1.0.0> which should return true, but
-doesn't. This only matters in comparisons, only for version numbers prior to
-C<1.4.0-rc1> (June 2006), and only when the compared versions are very close.
+Note that in the C<git.git> repository, C<v1.0.1> and C<v1.0.2> are
+lightweight tags that points respectively to C<v1.0.0a> and C<v1.0.0b>.
+As of Git::Repository 1.314, the comparison code internally converts
+C<v1.0.0a> and C<v1.0.0b> to their numerical equivalent before performing
+the comparison.
+
+Prior to C<1.4.0-rc1> (June 2006), compiling a development version of git
+would lead C<git --version> to output C<1.x-GIT> (with C<x> in C<0 .. 3>),
+which would make comparing versions that are very close a futile exercise.
 
 Other issues exist when comparing development version numbers with one
 another. For example, C<1.7.1.1> is greater than both C<1.7.1.1.gc8c07>
@@ -545,11 +573,11 @@ commit was merged.
 
 =head1 PLUGIN SUPPORT
 
-C<Git::Repository> intentionally has only few methods.
+L<Git::Repository> intentionally has only few methods.
 The idea is to provide a lightweight wrapper around git, to be used
 to create interesting tools based on Git.
 
-However, people will want to add extra functionality to C<Git::Repository>,
+However, people will want to add extra functionality to L<Git::Repository>,
 the obvious example being a C<log()> method that returns simple objects
 with useful attributes.
 
@@ -580,7 +608,17 @@ just prefix the fully qualified class name with a C<+>. For example:
 
 See L<Git::Repository::Plugin> about how to create a new plugin.
 
-=head1 OTHER PERL GIT WRAPPERS
+=head1 ACKNOWLEDGEMENTS
+
+Thanks to Todd Rinaldo, who wanted to add more methods to
+L<Git::Repository>, which made me look for a solution that would preserve
+the minimalism of L<Git::Repository>. The C<::Plugin> interface is what
+I came up with.
+
+=head1 OTHER PERL GIT WRAPPERS (a.k.a. SEE ALSO)
+
+(This section was written in June 2010. The other Git wrappers have
+probably evolved since that time.)
 
 A number of Perl git wrappers already exist. Why create a new one?
 
@@ -595,30 +633,32 @@ for when I looked at the existing Git wrappers on CPAN. They are the
 "rational" reason for writing my own (the real reason being of course
 "I thought it would be fun, and I enjoyed doing it").
 
-Even though it works well for me and others, C<Git::Repository> has its
+Even though it works well for me and others, L<Git::Repository> has its
 own shortcomings: it I<is> a I<low-level interface to Git commands>,
 anything complex requires you to deal with input/output handles,
 it provides no high-level interface to generate actual Git commands
-or process the output of commands (but have a look at the plugins),
-it doesn't fully work under Win32 yet, etc. One the following modules
-may therefore be better suited for your needs, depending on what you're
-trying to achieve.
+or process the output of commands (but have a look at the plugins), etc.
+One the following modules may therefore be better suited for your needs,
+depending on what you're trying to achieve.
 
 =head2 Git.pm
 
-Git.pm is not on CPAN. It is usually packaged with Git, and installed with
-the system Perl libraries. Not being on CPAN makes it harder to install
-in any Perl. It makes it harder for a CPAN library to depend on it.
+Git.pm was not on CPAN in 2010. It is packaged with Git, and installed
+with the system Perl libraries. Not being on CPAN made it harder to
+install in any Perl. It made it harder for a CPAN library to depend on it.
 
 It doesn't allow calling C<git init> or C<git clone>.
 
 The C<command_bidi_pipe> function especially has problems:
 L<http://kerneltrap.org/mailarchive/git/2008/10/24/3789584>
 
+The L<Git> module from git.git was packaged as a CPAN distribution by
+MSOUTH in June 2013.
 
 =head2 Git::Class
 
-Depends on Moose, which seems an unnecessary dependency for a simple
+L<Git::Class>
+depends on Moose, which seems an unnecessary dependency for a simple
 wrapper around Git. The startup penalty could become significant for
 command-line tools.
 
@@ -627,23 +667,38 @@ Although it supports C<git init> and C<git clone>
 porcelain commands, and provides no way to control bidirectional commands
 (such as C<git commit-tree>).
 
-
 =head2 Git::Wrapper
 
-Doesn't support streams or bidirectional commands.
+L<Git::Wrapper>
+doesn't support streams or bidirectional commands.
 
+=head2 Git::Sub
 
-=head1 AUTHOR
+(This description was added for completeness in May 2013.)
 
-Philippe Bruhat (BooK), C<< <book at cpan.org> >>
+L<Git::Sub> appeared in 2013, as a set of Git-specific L<System::Sub>
+functions. It provide a nice set of C<git::> functions, and has some
+limitations (due to the way L<System::Sub> itself works) which don't
+impact most Git commands.
+
+L<Git::Sub> doesn't support working with streams.
+
+=head2 Git::Raw
+
+(This description was added for completeness in September 2014,
+upon request of the author of L<Git::Raw>.)
+
+L<Git::Raw>
+provides bindings to L<libgit2|http://libgit2.github.com/>, a pure C
+implementation of the Git core methods. Most of the functions provided by
+libgit2 are available. If you have complex workflows, or even if speed is of
+the essence, this may be a more attractive solution than shelling out to git.
 
 =head1 BUGS
 
-Since version 1.17, C<Git::Repository> delegates the actual command
-execution to C<System::Command>. Win32 support for that module is
-currently very bad (the test suite hangs in a few places).
-If you'd like better Win32 support for C<Git::Repository>, help me improve
-C<System::Command>!
+Since version 1.17, L<Git::Repository> delegates the actual command
+execution to L<System::Command>, which has better support for Win32
+since version 1.100.
 
 Please report any bugs or feature requests to C<bug-git-repository at rt.cpan.org>, or through
 the web interface at L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=Git-Repository>.  I will be notified, and then you'll
@@ -655,7 +710,6 @@ You can find documentation for this module with the perldoc command.
 
     perldoc Git::Repository
 
-
 You can also look for information at:
 
 =over 4
@@ -678,17 +732,23 @@ L<http://search.cpan.org/dist/Git-Repository>
 
 =back
 
+=head1 BUGS
 
-=head1 ACKNOWLEDGEMENTS
+Please report any bugs or feature requests on the bugtracker website
+http://rt.cpan.org/NoAuth/Bugs.html?Dist=Git-Repository or by email to
+bug-git-repository@rt.cpan.org.
 
-Thanks to Todd Rinalo, who wanted to add more methods to
-C<Git::Repository>, which made me look for a solution that would preserve
-the minimalism of C<Git::Repository>. The C<::Plugin> interface is what
-I came up with.
+When submitting a bug or request, please include a test-file or a
+patch to an existing test-file that illustrates the bug or desired
+feature.
+
+=head1 AUTHOR
+
+Philippe Bruhat (BooK) <book@cpan.org>
 
 =head1 COPYRIGHT
 
-Copyright 2010-2011 Philippe Bruhat (BooK), all rights reserved.
+Copyright 2010-2014 Philippe Bruhat (BooK), all rights reserved.
 
 =head1 LICENSE
 
@@ -696,4 +756,3 @@ This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
 
 =cut
-
